@@ -9,6 +9,201 @@ use crate::options::*;
 use crate::util::OneWayOwned;
 use crate::writer::Writer;
 
+struct LegendData
+{
+	x: Coordinate,
+	y: Coordinate,
+	legend_options: Vec<LegendOption<String>>,
+	text_options: Vec<LabelOption<String>>,
+}
+
+impl LegendData
+{
+	fn write_out(&self, writer: &mut dyn Writer)
+	{
+		let w = writer;
+		write!(w, "set key at {},{}", self.x, self.y);
+
+		first_opt_default! {self.legend_options,
+			Placement(h, v) =>
+			{
+				w.write_str(match h
+				{
+					AlignLeft => " left",
+					AlignRight => " right",
+					_ => " center"
+				});
+				w.write_str(match v
+				{
+					AlignTop => " top",
+					AlignBottom => " bottom",
+					_ => " center"
+				});
+			},
+			_ =>
+			{
+				w.write_str(" right top");
+			}
+		}
+
+		first_opt_default! {self.legend_options,
+			Horizontal =>
+			{
+				w.write_str(" horizontal");
+			},
+			_ =>
+			{
+				w.write_str(" vertical");
+			}
+		}
+
+		first_opt_default! {self.legend_options,
+			Reverse =>
+			{
+				w.write_str(" reverse");
+			},
+			_ =>
+			{
+				w.write_str(" noreverse");
+			}
+		}
+
+		first_opt_default! {self.legend_options,
+			Invert =>
+			{
+				w.write_str(" invert");
+			},
+			_ =>
+			{
+				w.write_str(" noinvert");
+			}
+		}
+
+		first_opt! {self.legend_options,
+			Title(ref s) =>
+			{
+				w.write_str(" title \"");
+				w.write_str(s);
+				w.write_str("\"");
+			}
+		}
+
+		first_opt! {self.text_options,
+			Font(ref f, s) =>
+			{
+				w.write_str(" font \"");
+				w.write_str(f);
+				w.write_str(",");
+				w.write_str(&s.to_string()[..]);
+				w.write_str("\"");
+			}
+		}
+		first_opt! {self.text_options,
+			TextColor(ref s) =>
+			{
+				w.write_str(" textcolor rgb \"");
+				w.write_str(&s);
+				w.write_str("\"");
+			}
+		}
+		first_opt! {self.text_options,
+			TextAlign(a) =>
+			{
+				w.write_str(match a
+				{
+					AlignLeft => " Left",
+					AlignRight => " Right",
+					_ => ""
+				});
+			}
+		}
+
+		first_opt! {self.legend_options,
+			MaxRows(r) =>
+			{
+				write!(w, " maxrows {}", r as i32);
+			}
+		}
+
+		first_opt! {self.legend_options,
+			MaxCols(l) =>
+			{
+				write!(w, " maxcols {}", l as i32);
+			}
+		}
+
+		w.write_str("\n");
+	}
+
+	fn reset_state(&self, writer: &mut dyn Writer)
+	{
+		writer.write_str("unset key\n");
+	}
+}
+
+struct ArrowData
+{
+	x1: Coordinate,
+	y1: Coordinate,
+	x2: Coordinate,
+	y2: Coordinate,
+	plot_options: Vec<PlotOption<String>>,
+	tag: i32,
+}
+
+impl ArrowData
+{
+	fn write_out(&self, writer: &mut dyn Writer)
+	{
+		let w = writer;
+		write!(
+			w,
+			"set arrow {} from {},{} to {},{}",
+			self.tag, self.x1, self.y1, self.x2, self.y2
+		);
+
+		first_opt! {self.plot_options,
+			ArrowType(s) =>
+			{
+				w.write_str(match s
+				{
+					Open => "",
+					Closed => " empty",
+					Filled => " filled",
+					NoArrow => " nohead",
+				});
+			}
+		}
+
+		w.write_str(" size graph ");
+		first_opt_default! {self.plot_options,
+			ArrowSize(z) =>
+			{
+				write!(w, "{:.12e}", z);
+			},
+			_ =>
+			{
+				w.write_str("0.05");
+			}
+		}
+		w.write_str(",12");
+
+		AxesCommonData::write_color_options(w, &self.plot_options, Some("black"));
+		AxesCommonData::write_line_options(
+			w,
+			&self.plot_options,
+			GnuplotVersion { major: 0, minor: 0 },
+		);
+
+		w.write_str("\n");
+	}
+
+	fn reset_state(&self, writer: &mut dyn Writer)
+	{
+		writeln!(writer, "unset arrow {}", self.tag);
+	}
+}
+
 struct BorderOptions
 {
 	front: bool,
@@ -27,7 +222,7 @@ impl BorderOptions
 		}
 	}
 
-	fn write_out(&self, writer: &mut Writer, version: GnuplotVersion)
+	fn write_out(&self, writer: &mut dyn Writer, version: GnuplotVersion)
 	{
 		writer.write_str("set border ");
 		let mut f: i32 = 0;
@@ -50,6 +245,8 @@ pub struct Axes2D
 {
 	common: AxesCommonData,
 	border_options: BorderOptions,
+	arrows: Vec<ArrowData>,
+	legend: Option<LegendData>,
 }
 
 impl Axes2D
@@ -59,6 +256,8 @@ impl Axes2D
 		Axes2D {
 			common: AxesCommonData::new(),
 			border_options: BorderOptions::new(),
+			arrows: vec![],
+			legend: None,
 		}
 	}
 
@@ -123,42 +322,14 @@ impl Axes2D
 		options: &[PlotOption<&str>],
 	) -> &'l mut Self
 	{
-		{
-			let c = &mut self.common.commands as &mut Writer;
-			write!(c, "set arrow from {},{} to {},{}", x1, y1, x2, y2);
-
-			first_opt! {options,
-				ArrowType(s) =>
-				{
-					c.write_str(match s
-					{
-						Open => "",
-						Closed => " empty",
-						Filled => " filled",
-						NoArrow => " nohead",
-					});
-				}
-			}
-
-			c.write_str(" size graph ");
-			first_opt_default! {options,
-				ArrowSize(z) =>
-				{
-					write!(c, "{:.12e}", z);
-				},
-				_ =>
-				{
-					c.write_str("0.05");
-				}
-			}
-			c.write_str(",12");
-
-			let options: Vec<PlotOption<String>> = options.to_one_way_owned();
-			AxesCommonData::write_color_options(c, &options, Some("black"));
-			AxesCommonData::write_line_options(c, &options, GnuplotVersion { major: 0, minor: 0 });
-
-			c.write_str("\n");
-		}
+		self.arrows.push(ArrowData {
+			x1: x1,
+			y1: y1,
+			x2: x2,
+			y2: y2,
+			tag: self.arrows.len() as i32 + 1,
+			plot_options: options.to_one_way_owned(),
+		});
 		self
 	}
 
@@ -166,7 +337,7 @@ impl Axes2D
 	/// # Arguments
 	/// * `x` - X coordinate of the legend
 	/// * `y` - Y coordinate of the legend
-	/// * `label_options` - Array of LegendOption options
+	/// * `legend_options` - Array of LegendOption options
 	/// * `text_options` - Array of LabelOption options specifying the appearance of the plot titles. Valid options are:
 	///     * `Font`
 	///     * `TextColor`
@@ -177,121 +348,12 @@ impl Axes2D
 		text_options: &[LabelOption<&str>],
 	) -> &'l mut Self
 	{
-		{
-			let c = &mut self.common.commands as &mut Writer;
-
-			write!(c, "set key at {},{}", x, y);
-
-			first_opt_default! {legend_options,
-				Placement(h, v) =>
-				{
-					c.write_str(match h
-					{
-						AlignLeft => " left",
-						AlignRight => " right",
-						_ => " center"
-					});
-					c.write_str(match v
-					{
-						AlignTop => " top",
-						AlignBottom => " bottom",
-						_ => " center"
-					});
-				},
-				_ =>
-				{
-					c.write_str(" right top");
-				}
-			}
-
-			first_opt_default! {legend_options,
-				Horizontal =>
-				{
-					c.write_str(" horizontal");
-				},
-				_ =>
-				{
-					c.write_str(" vertical");
-				}
-			}
-
-			first_opt_default! {legend_options,
-				Reverse =>
-				{
-					c.write_str(" reverse");
-				},
-				_ =>
-				{
-					c.write_str(" noreverse");
-				}
-			}
-
-			first_opt_default! {legend_options,
-				Invert =>
-				{
-					c.write_str(" invert");
-				},
-				_ =>
-				{
-					c.write_str(" noinvert");
-				}
-			}
-
-			first_opt! {legend_options,
-				Title(s) =>
-				{
-					c.write_str(" title \"");
-					c.write_str(s);
-					c.write_str("\"");
-				}
-			}
-
-			first_opt! {text_options,
-				Font(f, s) =>
-				{
-					c.write_str(" font \"");
-					c.write_str(f);
-					c.write_str(",");
-					c.write_str(&s.to_string()[..]);
-					c.write_str("\"");
-				}
-			}
-			first_opt! {text_options,
-				TextColor(s) =>
-				{
-					c.write_str(" textcolor rgb \"");
-					c.write_str(s);
-					c.write_str("\"");
-				}
-			}
-			first_opt! {text_options,
-				TextAlign(a) =>
-				{
-					c.write_str(match a
-					{
-						AlignLeft => " Left",
-						AlignRight => " Right",
-						_ => ""
-					});
-				}
-			}
-
-			first_opt! {legend_options,
-				MaxRows(r) =>
-				{
-					write!(c, " maxrows {}", r as i32);
-				}
-			}
-
-			first_opt! {legend_options,
-				MaxCols(l) =>
-				{
-					write!(c, " maxcols {}", l as i32);
-				}
-			}
-
-			c.write_str("\n");
-		}
+		self.legend = Some(LegendData {
+			x: x,
+			y: y,
+			legend_options: legend_options.to_one_way_owned(),
+			text_options: text_options.to_one_way_owned(),
+		});
 		self
 	}
 
@@ -379,7 +441,7 @@ impl Axes2D
 	/// Additionally, error bars are attached to each data point in the X direction.
 	/// # Arguments
 	/// * `x` - x values
-	/// * `y` - y valuess
+	/// * `y` - y values
 	/// * `x_error` - Errors associated with the x value
 	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
@@ -445,7 +507,7 @@ impl Axes2D
 	/// Additionally, error bars are attached to each data point in the X direction.
 	/// # Arguments
 	/// * `x` - x values
-	/// * `y` - y valuess
+	/// * `y` - y values
 	/// * `x_error` - Errors associated with the x value
 	/// * `options` - Array of PlotOption controlling the appearance of the plot element. The relevant options are:
 	///     * `Caption` - Specifies the caption for this dataset. Use an empty string to hide it (default).
@@ -732,6 +794,44 @@ impl Axes2D
 		));
 		self
 	}
+
+	pub(crate) fn write_out(
+		&self, writer: &mut dyn Writer, auto_layout: bool, version: GnuplotVersion,
+	)
+	{
+		self.common.write_out_commands(writer, auto_layout, version);
+		self.border_options.write_out(writer, version);
+		let mut grid_axes = vec![];
+		if self.common.x_axis.grid
+		{
+			grid_axes.push(self.common.x_axis.axis);
+		}
+		if self.common.y_axis.grid
+		{
+			grid_axes.push(self.common.y_axis.axis);
+		}
+		if self.common.cb_axis.grid
+		{
+			grid_axes.push(self.common.cb_axis.axis);
+		}
+		self.common.write_grid_options(writer, &grid_axes, version);
+		for arrow in &self.arrows
+		{
+			arrow.write_out(writer);
+		}
+		self.legend.as_ref().map(|l| l.write_out(writer));
+		self.common.write_out_elements("plot", writer, version);
+	}
+
+	pub(crate) fn reset_state(&self, writer: &mut dyn Writer)
+	{
+		self.common.reset_state(writer);
+		for arrow in &self.arrows
+		{
+			arrow.reset_state(writer);
+		}
+		self.legend.as_ref().map(|l| l.reset_state(writer));
+	}
 }
 
 impl AxesCommonPrivate for Axes2D
@@ -748,37 +848,3 @@ impl AxesCommonPrivate for Axes2D
 }
 
 impl AxesCommon for Axes2D {}
-
-pub(crate) trait Axes2DPrivate
-{
-	fn write_out(&self, writer: &mut Writer, version: GnuplotVersion);
-}
-
-impl Axes2DPrivate for Axes2D
-{
-	fn write_out(&self, writer: &mut Writer, version: GnuplotVersion)
-	{
-		if self.common.elems.len() == 0
-		{
-			return;
-		}
-
-		self.common.write_out_commands(writer, version);
-		self.border_options.write_out(writer, version);
-		let mut grid_axes = vec![];
-		if self.common.x_axis.grid
-		{
-			grid_axes.push(self.common.x_axis.axis);
-		}
-		if self.common.y_axis.grid
-		{
-			grid_axes.push(self.common.y_axis.axis);
-		}
-		if self.common.cb_axis.grid
-		{
-			grid_axes.push(self.common.cb_axis.axis);
-		}
-		self.common.write_grid_options(writer, &grid_axes, version);
-		self.common.write_out_elements("plot", writer, version);
-	}
-}
